import shutil


async def load_disks(hub):
    # partitions are called "slices" on SunOS, find out about the underlying hardware
    disks = []
    ssds = []

    iostat = shutil.which("iostat") or "/usr/bin/iostat"
    ret = await hub.exec.cmd.run([iostat, "-x"])
    # extended device statistics
    # device    r/s    w/s   kr/s   kw/s wait actv wsvc_t asvc_t  %w  %b
    # cmdk0     1.5   21.7   68.5  214.2  0.0  0.0    0.4    0.3   0   0

    # Skip the first two lines
    for line in ret.stdout.splitlines()[2:]:
        disks.append(line.split()[0])

    if disks:
        hub.grains.GRAINS.disks = sorted(disks)

    # TODO how to distinguish between SSDs and HDDs?
    if ssds:
        hub.grains.GRAINS.SSDs = sorted(ssds)



import shutil


async def load_default_gateway(hub):
    """
    Populates grains which describe whether a server has a default gateway
    configured or not. Uses `netstat -rn4 ` and `netstat -rn6` and looks
    for a `default` at the beginning of any line. Assuming the standard
    `default via <ip>` format for default gateways, it will also parse out the
    ip address of the default gateway, and put it in ip4_gw or ip6_gw.

    If the `netstat` command is unavailable, no grains will be populated.

    Currently does not support multiple default gateways. The grains will be
    set to the first default gateway found.

    List of grains:

        ip4_gw: True  # ip/True/False if default ipv4 gateway
        ip6_gw: True  # ip/True/False if default ipv6 gateway
        ip_gw: True   # True if either of the above is True, False otherwise
    """
    netstat = shutil.which("netstat")

    if netstat:
        for protocol in (4, 6):
            ret = await hub.exec.cmd.run(
                [netstat, "-r", "-f", "inet" if protocol is 4 else "inet6"]
            )
            for line in ret.stdout.splitlines():
                if line.startswith("default"):
                    hub.grains.GRAINS[f"ip{protocol}_gw"] = line.split()[1]
                    break
            else:
                hub.grains.GRAINS[f"ip{protocol}_gw"] = False

        hub.grains.GRAINS.ip_gw = bool(
            hub.grains.GRAINS.ip4_gw or hub.grains.GRAINS.ip6_gw
        )



"""
TODO These grains are collected in salt, but some work regarding minion and master confs
needs to be done before they can be useful here
"""

import os


async def load_salt_path(hub):
    try:
        import salt

        hub.grains.GRAINS.saltpath = os.path.dirname(os.path.abspath(salt.__file__))
    except (ImportError, ModuleNotFoundError):
        hub.grains.GRAINS.saltpath = "unknown"


async def load_salt_version(hub):
    try:
        from salt.version import __version__, __version_info__

        hub.grains.GRAINS.saltversion = __version__
        hub.grains.GRAINS.saltversioninfo = __version_info__
    except (ImportError, ModuleNotFoundError):
        hub.grains.GRAINS.saltversion = "unknown"
        hub.grains.GRAINS.saltversioninfo = "unknown"



import getpass
import pwd


async def load_console_user(hub):
    hub.grains.GRAINS.console_username = getpass.getuser()
    hub.grains.GRAINS.console_user = pwd.getpwnam(
        hub.grains.GRAINS.console_username
    ).pw_uid



# -*- coding: utf-8 -*-
from typing import Any, Dict, List


async def sig_get(hub, string: str, clean: bool = True) -> str:
    pass


async def sig_records(
    hub, rec_type: int = None, fields: str or List[str] = None, clean: bool = True
) -> List[Dict[str, Any]]:
    pass



async def load_num_gpus(hub):
    # TODO figure out how to get gpu info
    # get the model and vendor for each GPU
    hub.grains.GRAINS.gpus = {}
    hub.grains.GRAINS.num_gpus = len(hub.grains.GRAINS.gpus)



import re
import shutil


async def load_bios_data(hub):
    # A simple value cleaner for regex matches
    clean = lambda x: x.strip().replace("'", "")

    # Run the commands that generate data for bios info
    prtconf = (
        await hub.exec.cmd.run([shutil.which("prtconf") or "/usr/sbin/prtconf", "-vp"])
    ).stdout
    prtdiag = (
        await hub.exec.cmd.run([shutil.which("prtdiag") or "/usr/sbin/prtdiag", "-v"])
    ).stdout
    virtinfo = (
        await hub.exec.cmd.run([shutil.which("virtinfo") or "/usr/sbin/virtinfo", "-a"])
    ).stdout

    # Load biosversion and biosreleasedate
    match = re.search(r"(?im)version:\s*\'OBP\s+(\S+)\s+(\S+)", prtconf) or re.search(
        r"(?im)System\s+PROM\s+revisions.*\n(?:Version\n)?-+\nOBP\s+(\S+)\s+(\S+)",
        prtdiag,
    )
    hub.grains.GRAINS.biosversion = (
        clean(match.group(1))
        if match
        else await hub.exec.smbios.get("bios-version") or "unknown"
    )
    hub.grains.GRAINS.biosreleasedate = (
        clean(match.group(2))
        if match
        else await hub.exec.smbios.get("bios-release-date") or "unknown"
    )

    # Load manufacturer
    match = re.search(r"(?im)^\s*System\s+Configuration:\s*(.*)(?=sun)", prtdiag)
    hub.grains.GRAINS.manufacturer = (
        clean(match.group(1))
        if match
        else await hub.exec.smbios.get("system-manufacturer") or "unknown"
    )

    # load productname
    match = re.search(
        r"(?im)^[^\S\r\n]*(?:banner|product)-name:[^\S\r\n]*(.*)", prtconf
    ) or re.search(
        r"(?im)^\s*System\s+Configuration:\s*.*?sun\d\S+[^\S\r\n]*(.*)", prtdiag
    )
    hub.grains.GRAINS.productname = (
        clean(match.group(1))
        if match
        else await hub.exec.smbios.get("system-product-name") or "unknown"
    )

    # Load serial number
    match = (
        re.search(r"(?i)chassis-sn:\s*(\S+)", prtconf)
        or re.search(r"(?im)Chassis\s+Serial\s+Number\n-+\n(\S+)", prtdiag)
        or re.search(r"(?i)Chassis\s+Serial#:\s*(\S+)", virtinfo)
    )
    hub.grains.GRAINS.serialnumber = (
        clean(match.group(1))
        if match
        else (
            await hub.exec.smbios.get("baseboard-serial-number")
            or await hub.exec.smbios.get("chassis-serial-number")
            or await hub.exec.smbios.get("system-serial-number")
        )
        or "unkown"
    )

    # Load system firmware
    match = re.search(r"(?i)Sun\s+System\s+Firmware\s+(\S+)\s+(\S+)", prtdiag)
    if match:
        hub.grains.GRAINS.systemfirmware = match.group(1)
        hub.grains.GRAINS.systemfirmwaredate = match.group(2)

    # Load uuid
    match = re.search(r"(?i)Domain\s+UUID:\s+(\S+)", virtinfo)
    hub.grains.GRAINS.uuid = (
        clean(match.group(1))
        if match
        else await hub.exec.smbios.get("system-uuid") or "unknown"
    )



import shutil
import os


async def load_init_system(hub):
    if any(shutil.which(smf_cmd) for smf_cmd in ("svcs", "svcadm", "svcprop")) or any(
        os.path.exists(smf_path)
        for smf_path in ("/usr/bin/svcs", "/usr/sbin/svcadm", "/usr/bin/svcprop")
    ):
        # For solaris 10 and 11
        hub.grains.GRAINS.init = "smf"
    else:
        hub.grains.GRAINS.init = "unknown"

    # TODO it was sys v for solaris < 7, which files or commands should be checked for
    # TODO Solaris 8 and 9 it will be inittab, which files or commands should be checked for
